/*
    AQTRONIX C++ Library
    Copyright 2003-2008 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// IPRange.cpp: implementation of the CIPRange class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "IPRange.h"

#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* Changelog
 * 2008.09.23 Moved XOR,AND,OR to CIPAddressTemplate (for IPv6)
 * 2008.09.22 Added ':' to allowed range characters (for IPv6)
 * 2008.01.21 Fixed bug missing allowed character '-' in IsValidIPRange()
 * 2007.05.04 Added GetSubnetID()
 * 2005.10.25 Did some int -> unsigned char conversions for performance
 * 2005.10.02 Changed CString parameter of IsInRange() to LPCTSTR
 *            Moved ConvertCIDRToSubnet() to class CIPAddress
 * 2005.08.03 Added range format 10.0.0.1-10.0.0.2
 * 2005.07.05 Code review + use asignment operator of CIPAddress class (=)
 * 2004.01.16 First version of class finished
 * 2003.10.15 Created class for MailKnight
 */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIPRange::CIPRange()
{
	Max.SetMax();
}

CIPRange::CIPRange(CIPAddress& StartIP, CIPAddress& EndIP)
{
	Max.SetMax();
	SetIPRange(StartIP,EndIP);
}

CIPRange::CIPRange(CString IPRange)
{
	Max.SetMax();
	SetIPRange(IPRange);
}

CIPRange::~CIPRange()
{

}

CString CIPRange::GetStartIP() const
{
	return StartIP.ToString();
}

CString CIPRange::GetEndIP() const
{
	return EndIP.ToString();
}

CString CIPRange::GetSubnetMask() const
{
	return SubnetMask.ToString();
}

CString CIPRange::GetSubnetID(CString& IP, CString& CIDR)
{
	CIPRange range(IP + "/" + CIDR);
	return range.GetStartIP() + "/" + CIDR;
}

bool CIPRange::IsValidIPRange(CString &IP)
{
	/*
	 * returns true if it contains only numbers, dots and '*' or '/' and cannot be empty
	 */
	if(IP.GetLength()>0){
		for(int i=0;i<IP.GetLength();i++){
			if(IP[i] != '.' && (IP[i]<48 || IP[i]>57) && IP[i]!='*' && IP[i]!='/' && IP[i]!='-' && IP[i]!=':'){ //colon for IPv6
				return false;
			}
		}
	}else{
		return false;
	}
	return true;
}

bool CIPRange::IsInRange(CIPAddress& IP)
{
	if(IP<StartIP || IP>EndIP){
		return false;
	}else{
		return true;
	}
}

bool CIPRange::SetIPRange(CIPAddress &StartIP, CIPAddress &EndIP)
{
	//set start en end IP
	CIPRange::StartIP = StartIP;
	CIPRange::EndIP = EndIP;

	//set subnetmask
	CIPAddress SubnetID;
	SubnetID = StartIP.XOR(EndIP);
	SubnetMask = SubnetID.XOR(Max);

	return true;
}

bool CIPRange::SetIPRange(CString IPRange)
{
	int possep;

	//remove everything after space (comments...)
	possep = IPRange.Find(' ');
	if(possep>-1){
		IPRange = IPRange.Left(possep);
	}

	//is valid ip range?
	if(IsValidIPRange(IPRange)){
		possep = IPRange.Find('/');
		if(possep==-1){
			possep = IPRange.Find('-');
			if(possep==-1){
				//format 10.*.*.*
				while(IPRange.Find("**")!=-1){
					IPRange.Replace("**","*");
				}
				//startip
				CString RangeStart(IPRange);
				RangeStart.Replace("*","0");
				StartIP = RangeStart;
				//endip
				IPRange.Replace("*","255");
				EndIP = IPRange;
			}else{
				//format 10.0.0.1-10.0.0.2
				//startip
				StartIP = IPRange.Left(possep);
				//endip
				if(IPRange.GetLength()>possep){
					EndIP = IPRange.Mid(possep+1);
				}else{
					EndIP = StartIP;
				}
			}
			//subnetmask
			CIPAddress SubnetID;
			SubnetID = StartIP.XOR(EndIP);
			SubnetMask = SubnetID.XOR(Max);
		}else{
			//format 10.0.0.0/24
			unsigned char subnet;
			CString strSubnet = IPRange.Mid( (possep+1<IPRange.GetLength())?possep+1:possep );
			subnet = CStringHelper::Safe_AsciiToUChar(strSubnet);
			SubnetMask.SetIP(CIPAddress::ConvertCIDRToSubnet(subnet));
			//startip
			CIPAddress IP(IPRange.Left((possep>0?possep:possep-1)));
			StartIP = IP.AND(SubnetMask);
			//endip
			CIPAddress HostID;
			HostID = SubnetMask.XOR(Max);
			EndIP = StartIP.OR(HostID);
		}
		return true;
	}else{
		return false;
	}
}

bool CIPRange::IsInRange(LPCTSTR IP)
{
	CIPAddress IPa;
	if(IPa.SetIP(IP)){
		if(IPa<StartIP || IPa>EndIP){
			return false;
		}else{
			return true;
		}
	}else{
		return false;
	}
}

CString CIPRange::ToString() const
{
	return StartIP.ToString() + "-" + EndIP.ToString() ;
}
